<?php

declare(strict_types=1);

namespace mirzaev\gavno\models;

// Фреймворк PHP
use mirzaev\minimal\model;

// Фреймворк ArangoDB
use mirzaev\arangodb\connection as arangodb;

use exception;

/**
 * Ядро моделей
 *
 * @package mirzaev\gavno\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class core extends model
{
	/**
	 * Соединение с базой данных ArangoDB
	 */
	protected static arangodb $arangodb;

	/**
	 * Путь до файла с настройками подключения к базе данных ArangoDB
	 */
	final public const ARANGODB = '../settings/arangodb.php';

	/**
	 * Конструктор
	 *
	 * @param bool $initialize Инициализировать контроллер?
	 * @param ?arangodb $arangodb Инстанция соединения с базой данных ArangoDB
	 */
	public function __construct(bool $initialize = true, ?arangodb $arangodb = null)
	{
		if ($initialize) {
			// Запрошена инициализация

			if (isset($arangodb)) {
				// Получена инстанция соединения с базой данных

				// Запись и инициализация соединения с базой данных
				$this->__set('arangodb', $arangodb);
			} else {
				// Не получена инстанция соединения с базой данных

				// Инициализация соединения с базой данных по умолчанию
				$this->__get('arangodb');
			}
		}
	}

	/**
	 * Записать свойство
	 *
	 * @param string $name Название
	 * @param mixed $value Значение
	 */
	public function __set(string $name, mixed $value = null): void
	{
		match ($name) {
			'arangodb' => (function () use ($value) {
				if ($this->__isset('arangodb')) {
					// Свойство уже было инициализировано

					// Выброс исключения (неудача)
					throw new exception('Запрещено реинициализировать соединение с базой данных ($this->arangodb)', 500);
				} else {
					// Свойство ещё не было инициализировано

					if ($value instanceof arangodb) {
						// Передано подходящее значение

						// Запись свойства (успех)
						self::$arangodb = $value;
					} else {
						// Передано неподходящее значение

						// Выброс исключения (неудача)
						throw new exception('Соединение с базой данных ($this->arangodb) должен быть инстанцией mirzaev\arangodb\connection', 500);
					}
				}
			})(),
			default => parent::__set($name, $value)
		};
	}

	/**
	 * Прочитать свойство
	 *
	 * @param string $name Название
	 *
	 * @return mixed Содержимое
	 */
	public function __get(string $name): mixed
	{
		return match ($name) {
			'arangodb' => (function () {
				if (!$this->__isset('db')) {
					// Свойство не инициализировано

					// Инициализация значения по умолчанию исходя из настроек
					$this->__set('arangodb', new arangodb(require static::ARANGODB));
				}

				return self::$arangodb;
			})(),
			default => parent::__get($name)
		};
	}

	/**
	 * Проверить свойство на инициализированность
	 *
	 * @param string $name Название
	 */
	public function __isset(string $name): bool
	{
		return match ($name) {
			default => parent::__isset($name)
		};
	}

	/**
	 * Удалить свойство
	 *
	 * @param string $name Название
	 */
	public function __unset(string $name): void
	{
		match ($name) {
			default => parent::__isset($name)
		};
	}


	/**
	 * Статический вызов
	 *
	 * @param string $name Название
	 * @param array $arguments Параметры
	 */
	public static function __callStatic(string $name, array $arguments): mixed
	{
		match ($name) {
			default => throw new exception("Не найдено свойство или функция: $name", 500)
		};
	}
}
